package be.nikiroo.utils.main;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.net.ssl.SSLException;

import be.nikiroo.utils.Cache;
import be.nikiroo.utils.CacheMemory;
import be.nikiroo.utils.CryptUtils;
import be.nikiroo.utils.Downloader;
import be.nikiroo.utils.StringUtils;
import be.nikiroo.utils.test.TestLauncher;

/**
 * Tests for nikiroo-utils.
 * 
 * @author niki
 */
public class Test extends TestLauncher {
	private String longString;

	/**
	 * Start the tests.
	 * 
	 * @param args
	 *            the arguments (which are passed as-is to the other test
	 *            classes)
	 * @throws SSLException
	 */
	public Test(String[] args) throws SSLException {
		super("Nikiroo-utils", args);

		// setDetails(true);

		// 1 mega chararcters
		StringBuilder builder = new StringBuilder();
		int nbr = 100; // nbr of times to process it
		for (int i = 0; i < 200; i++) {
			// 50 chars
			builder.append("Ceci est une longue chaîne de caractères 1234567");
		}
		longString = builder.toString();
		byte[] longBytes = StringUtils.getBytes(longString);
		System.out.println("Will work with: " + longBytes.length + " bytes ("
				+ StringUtils.formatNumber(longBytes.length) + "bytes)");

		long start = new Date().getTime();
		CryptUtils cryptUtils = new CryptUtils("some key");

		long setup = new Date().getTime();
		info("Setup", start, setup);

		byte[] encrypted = null;
		for (int i = 0; i < nbr; i++) {
			encrypted = cryptUtils.encrypt(longBytes);
		}

		long enc = new Date().getTime();
		info("Encryption", setup, enc);

		byte[] decrypted = null;
		for (int i = 0; i < nbr; i++) {
			decrypted = cryptUtils.decrypt(encrypted);
		}

		long dec = new Date().getTime();
		info("Decryption", enc, dec);

		// TODO: test cache and downloader
		Cache cache = null;
		CacheMemory memcache = null;
		Downloader downloader = null;

		// To include the sources:
		img2aa siu;
		justify ssu;
		bridge aa;
	}

	static void info(String name, long start, long stop) {
		long time = (stop - start);
		System.out.println(String.format("%s: %d ms (%.2f Mb/sec)", name, time,
				(1000f / time)));
	}

	/**
	 * Main entry point of the program.
	 * 
	 * @param args
	 *            the arguments passed to the {@link TestLauncher}s.
	 * @throws SSLException
	 */
	static public void main(String[] args) throws SSLException {
		// System.exit(new Test(args).launch());
		setList("test", Arrays.asList(new String[] { "un", "deux" }));
		List<String> list = getList("\"A\\A\", \"k\\\"l\"");
		System.out.println("Got:");
		if (list == null) {
			System.out.println("NULL (bad format)");
		} else {
			for (String s : list) {
				System.out.println("\t[" + s + "]");
			}
		}
	}

	/**
	 * Return the value associated to the given id as a list of values if it is
	 * found and can be parsed.
	 * 
	 * @param id
	 *            the id of the value to get
	 * 
	 * @return the associated list, empty if the value is empty, NULL if it is
	 *         not found or cannot be parsed as a list
	 */
	static public List<String> getList(String id) {
		List<String> list = new ArrayList<String>();
		String raw = getString(id);
		try {
			boolean inQuote = false;
			boolean prevIsBackSlash = false;
			StringBuilder builder = new StringBuilder();
			for (int i = 0; i < raw.length(); i++) {
				char car = raw.charAt(i);
				// System.out.println("car: [" + car + "]");

				if (prevIsBackSlash) {
					builder.append(car);
					prevIsBackSlash = false;
				} else {
					switch (car) {
					case '"':
						if (inQuote) {
							list.add(builder.toString());
							builder.setLength(0);
						}

						inQuote = !inQuote;
						break;
					case '\\':
						prevIsBackSlash = true;
						break;
					case ' ':
					case '\n':
					case '\r':
						if (inQuote) {
							builder.append(car);
						}
						break;

					case ',':
						if (!inQuote) {
							break;
						}
						// continue to default
					default:
						if (!inQuote) {
							// Bad format!
							return null;
						}

						builder.append(car);
						break;
					}
				}

			}

			if (inQuote || prevIsBackSlash) {
				// Bad format!
				return null;
			}
		} catch (Exception e) {
			return null;
		}

		return list;
	}

	/**
	 * Set the value associated to the given id as a list of values.
	 * 
	 * @param id
	 *            the id of the value to set
	 * @param list
	 *            the new list of values
	 */
	static public void setList(String id, List<String> list) {
		StringBuilder builder = new StringBuilder();
		for (String item : list) {
			if (builder.length() > 0) {
				builder.append(", ");
			}
			builder.append('"')//
					.append(item.replace("\\", "\\\\").replace("\"", "\\\""))//
					.append('"');
		}

		setString(id, builder.toString());
	}

	static private String getString(String id) {
		return id;
	}

	static private void setString(String id, String value) {
		System.out.println("setString " + id + " -> [" + value + "]");
	}

}
